import _isNumber from "lodash-es/isNumber";
import _some from "lodash-es/some";
import _every from "lodash-es/every";
import _isArray from "lodash-es/isArray";
import _isInteger from "lodash-es/isInteger";
import _isString from "lodash-es/isString";
import _noop from "lodash-es/noop";
import { TypeAction } from './keyframe-list/types';
import { simple } from './simple';
import { SYMBOL_STATE } from '@escapace/fluent';
import { SYMBOL_KEYFRAME_LIST } from './types';
const NODE_ENV = process.env.NODE_ENV;

function ok(condition, message) {
  if (!condition) {
    const error = new Error(message);
    error.name = 'Assertion Error';
    throw error;
  }
}

const assertions = {
  ok,

  string(value) {
    ok(_isString(value), 'TODO:');
  },

  list(value) {
    const state = simple(value)[SYMBOL_STATE];
    ok(state.type === SYMBOL_KEYFRAME_LIST && !state.isEmpty, 'TODO:');
  },

  iterations(value) {
    if (_isArray(value)) {
      ok(_every(value, num => _isInteger(num)), 'TODO:');
    } else {
      ok(_isInteger(value), 'TODO:');
    }
  },

  keyframeName(value, // eslint-disable-next-line @typescript-eslint/no-explicit-any
  log) {
    ok(_isString(value) && !_some(log, action => action.type === TypeAction.Keyframes && action.payload.name === value), 'TODO:');
  },

  number(value) {
    ok(_isNumber(value), 'TODO:');
  }

};
export const assert = NODE_ENV !== 'production' ? assertions : /*#__PURE__*/Object.assign({}, ... /*#__PURE__*/Object.keys(assertions).map(name => ({
  [name]: _noop
})));
//# sourceMappingURL=assert.js.map
